local addon = LibStub("AceAddon-3.0"):GetAddon("Ouro Loot")
local plugin = addon:NewModule("CSV")

local pairs, ipairs, tostring, tonumber = pairs, ipairs, tostring, tonumber
local tconcat, wipe = table.concat, table.wipe
local GUI = LibStub("AceGUI-3.0")

plugin.option_defaults = {
	['format_loot'] = "DateStamp,PlayerName,ItemName,ItemCount,Disposition,Extra",
	['format_raiders'] = "DateStamp,Boss,PlayerName,Status",
	['date_loot'] = "DAY",
	['date_raiders'] = "%Y-%m-%d %H:%M",
	['sep_loot'] = ":",
	['sep_raiders'] = ",",
}

plugin.skiptoken = "\001SKIP"

--function plugin:OnFirstTime() end

function plugin:use_basic_day_p (whichtab)
	return self.opts['date_'..whichtab] == "DAY"   -- localize this someday
end

local funccache
local function buildcache (which, fmt)
	if not funccache then
		funccache = { ['EMPTY'] = function() return "" end }
	end
	local empty = funccache.EMPTY
	for j,tag in ipairs(fmt) do
		funccache[which..'_'..tag] = plugin[which..'_'..tag] or empty
	end
end


local function ShowFieldEditor (whichtab)
	assert(whichtab == 'loot' or whichtab == 'raiders')
	local keepformat = OuroLootCSV_opts['format_'..whichtab]
	local dateformat = OuroLootCSV_opts['date_'..whichtab]

	local W = GUI:Create("Window")
	-- This is a horrible way to loop over an ellipsis.  Good thing it's
	-- called only rarely.
	for _,r in ipairs{W.frame:GetRegions()} do
		if r --and r.IsObjectType
		   and r:IsObjectType("Texture")
		   and r:GetTexture() == [[Interface\Tooltips\UI-Tooltip-Background]]
		then
			r:SetVertexColor(0,0,0,1)  -- try to make the background opaque
			break
		end
	end
	W:SetTitle("Field Editor")
	W:SetLayout("List")
	W:SetStatusTable{width=700,height=500}  -- for now, same as default values
	W:EnableResize(false)
	W:SetCallback("OnClose", function(_w)
		if whichtab == 'loot' then
			addon:OpenMainDisplayToTab "CSV Loot"
		elseif whichtab == 'raiders' then
			addon:OpenMainDisplayToTab "CSV Attendance"
		end
	end)

	-- Top half or so for the fixed text.
	local top_container = GUI:Create("SimpleGroup")
	top_container:SetFullWidth(true)
	top_container:SetHeight(0.55 * W.status.height)
	top_container:SetLayout("Fill")
	local text = GUI:Create("Label")
	text:SetFullWidth(true)
	text:SetFontObject(GameFontNormal)
	text:SetText(plugin['helptext_'..whichtab])
	local top = GUI:Create("ScrollFrame")
	top:SetLayout("Flow")
	top:AddChild(text)
	top_container:AddChild(top)

	-- Bottom chunk for the editing lines.
	local bottom_container = GUI:Create("SimpleGroup")
	bottom_container:SetFullWidth(true)
	--bottom_container:SetHeight(0.25 * W.status.height)
	bottom_container:SetLayout("Flow")
	bottom_container:PauseLayout()

	local label = GUI:Create("Label")
	label:SetRelativeWidth(0.10)
	label:SetText("Format:")
	label:SetFontObject(GameFontNormalLarge)
	label:SetColor(30/255,0xad/255,1)   -- same blue as "+" in help markup
	local editbox = GUI:Create("EditBox")
	editbox:SetRelativeWidth(0.86)
	editbox:SetLabel("Use commas as separators here!  Spaces are fine and will be stripped.  Press Enter or click Okay when finished.")
	editbox:SetText(keepformat)
	editbox:SetCallback("OnEnterPressed", function(_e,event,value)
		keepformat = value
		_e.editbox:ClearFocus()
	end)
	--[[
	local spacer = GUI:Create("Spacer")
	spacer:SetFullWidth(true)
	spacer:SetHeight(5)]]
	bottom_container:AddChildren (label, editbox)
	--bottom_container:AddChildren (spacer, label, editbox)

	label = GUI:Create("Label")
	label:SetRelativeWidth(0.10)
	label:SetText("Date/Time:")
	label:SetFontObject(GameFontNormalLarge)
	label:SetColor(30/255,0xad/255,1)   -- same blue as "+" in help markup
	editbox = GUI:Create("EditBox")
	editbox:SetRelativeWidth(0.86)
	editbox:SetLabel("Either a strftime-style format string, or the word DAY for a simple date.  Press Enter or click Okay when finished.")
	editbox:SetText(dateformat)
	editbox:SetCallback("OnEnterPressed", function(_e,event,value)
		dateformat = value
		_e.editbox:ClearFocus()
	end)
	--[[
	spacer = GUI:Create("Spacer")
	spacer:SetFullWidth(true)
	spacer:SetHeight(5)]]


	local okay = GUI:Create("Button")
	okay:SetRelativeWidth(0.18)
	okay:SetText("Save & Close")
	okay:SetCallback("OnClick", function(_o)
		OuroLootCSV_opts['format_'..whichtab] = keepformat:gsub("%s","")
		OuroLootCSV_opts['date_'..whichtab] = dateformat
		funccache = nil
		W:Hide()
	end)
	local cancel = GUI:Create("Button")
	cancel:SetRelativeWidth(0.28)
	cancel:SetText("Discard Changes & Close")
	cancel:SetCallback("OnClick", function(_c)
		W:Hide()
	end)
	--bottom_container:AddChildren (spacer, label, editbox, okay, cancel)
	bottom_container:AddChildren (label, editbox, okay, cancel)
	bottom_container:ResumeLayout()

	W:PauseLayout()
	W:AddChild(top_container)
	W:AddChild(bottom_container)
	W:ResumeLayout()
	W:ApplyStatus()
	W:Show()
end


local function loot_generator (_, loot, last_printed, generated, cache)
	local line = {}
	local SKIP = plugin.skiptoken
	local sep = OuroLootCSV_opts.sep_loot
	local format = { strsplit(",", OuroLootCSV_opts.format_loot) }

	for j = 1, #format do
		format[j] = format[j]:lower()
	end
	buildcache('loot',format)

	for i = last_printed+1, #loot do
		local e = loot[i]

		if e.kind == 'loot' then
			for j,tag in ipairs(format) do
				line[j] = funccache['loot_'..tag](plugin, e, tag)
			end

		elseif e.kind == 'boss' then
			line[1] = plugin['boss_'..e.reason] and
				plugin['boss_'..e.reason](plugin, e) or SKIP

		elseif e.kind == 'time' then
			line[1] = plugin['time'] and plugin['time'](plugin, e) or SKIP

		end

		-- If any of the tag functions returned SKIP, do so.
		local do_line = true
		for j = 1, #format do
			if line[j] == SKIP then
				do_line = false
				break
			end
			line[j] = line[j] or ""
		end
		if do_line then
			cache[#cache+1] = tconcat (line, sep)
		end
		wipe(line)
	end

	return #cache > 0
end

local function loot_specials (_, editbox, container, mkbutton)
	local sepedit = mkbutton("EditBox", nil, OuroLootCSV_opts.sep_loot,
		[[Actual separator to use for "commas".  Keep in mind that commas themselves show up in item names.]])
	sepedit:SetFullWidth(true)
	sepedit:SetLabel("Loot Fields Output Separator:")
	sepedit:SetCallback("OnEnterPressed", function(_s,event,value)
		OuroLootCSV_opts.sep_loot = value
		_s.editbox:ClearFocus()
	end)
	container:AddChild(sepedit)

	local b = mkbutton("Fields...",
		[[Which fields to include on a loot line.  Closing the fields editor reopens this window.]])
	b:SetFullWidth(true)
	b:SetCallback("OnClick", function(_b)
		addon.display:Hide()
		ShowFieldEditor('loot')
	end)
	container:AddChild(b)
end

plugin:register_text_generator ("csvloot", [[CSV Loot]], [[Comma-separated value dump]], loot_generator, loot_specials)



local function raiders_generator (_, loot, last_printed, generated, cache)
	local line = {}
	local SKIP = plugin.skiptoken
	local sep = OuroLootCSV_opts.sep_raiders
	local format = { strsplit(",", OuroLootCSV_opts.format_raiders) }

	for j = 1, #format do
		format[j] = format[j]:lower()
	end
	buildcache('raiders',format)

	for i = last_printed+1, #loot do
		local e = loot[i]

		if e.kind == 'boss' then
			for player,playerinfo in pairs(e.raidersnap) do
				for j,tag in ipairs(format) do
					line[j] = funccache['raiders_'..tag](plugin, e, tag, player, playerinfo)
				end

				-- If any of the tag functions returned SKIP, do so.
				local do_line = true
				for k = 1, #format do
					if line[k] == SKIP then
						do_line = false
						break
					end
					line[k] = line[k] or ""
				end
				if do_line then
					cache[#cache+1] = tconcat (line, sep)
				end
				wipe(line)
			end

		elseif e.kind == 'time' then
			--line[1] = plugin['time'] and plugin['time'](plugin, e) or SKIP
			if plugin['time'] then plugin['time'](plugin, e) end

		end

	end

	return #cache > 0
end

local function raiders_specials (_, editbox, container, mkbutton)
	local sepedit = mkbutton("EditBox", nil, OuroLootCSV_opts.sep_raiders,
		[[Actual separator to use for "commas".  Keep in mind that commas themselves show up in encounter names.]])
	sepedit:SetFullWidth(true)
	sepedit:SetLabel("Attendance Fields Output Separator:")
	sepedit:SetCallback("OnEnterPressed", function(_s,event,value)
		OuroLootCSV_opts.sep_raiders = value
		_s.editbox:ClearFocus()
	end)
	container:AddChild(sepedit)

	local b = mkbutton("Fields...",
		[[Which fields to include on a raiders line.  Closing the fields editor reopens this window.]])
	b:SetFullWidth(true)
	b:SetCallback("OnClick", function(_b)
		addon.display:Hide()
		ShowFieldEditor('raiders')
	end)
	container:AddChild(b)
end

plugin:register_text_generator ("csvatt", [[CSV Attendance]], [[Comma-separated value dump]], raiders_generator, raiders_specials)

-- vim:noet
